/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this code; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  */

/* Copyright (C) 2002 Ian Zimmerman <itz@speakeasy.org> */

static const char rcsid[] = "$Id: my_color_selection.cc,v 1.3 2002/09/09 23:03:05 itz Exp $ ";

#include "my_color_selection.hh"
#include "my_clist.hh"
#include "show_dialog.hh"
#include "xcolorsel.hh"
#include "search_dialog.hh"

#include <gtk--/clist.h>
#include <gtk--/main.h>
#include <gdk--/color.h>
#include <gtk--/dialog.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/label.h>
#include <gtk--/box.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtkbox.h>

#include <algorithm>

static const char about[] = "xcolorsel-gtk " VERSION " by Ian Zimmerman.  Covered by the GPL";

void
my_color_selection::quit_handler()
{
  get_parent()->destroy();
}

void
my_color_selection::grab_handler()
{
  p_echo->set_text(about);
  xcolorsel_window* parent = static_cast<xcolorsel_window*>(get_parent());
  parent->start_grab_loop();
}

void
my_color_selection::show_handler()
{
  p_echo->set_text(about);
  gdouble colors[4];
  
  get_color(colors);
  modal = new show_dialog(colors[0], colors[1], colors[2]);
  modal->set_transient_for(static_cast<Gtk::Window&>(*get_parent()));
  end_modal_handler_id = modal->destroy.connect
    (SigC::slot(this, &my_color_selection::end_modal_handler));
}

void
my_color_selection::end_modal_handler()
{
  end_modal_handler_id.disconnect();
  delete modal;
  modal = 0;
}

void
my_color_selection::search_handler()
{
  p_echo->set_text(about);
  modal = new search_dialog(search_string.c_str());
  modal->set_transient_for(static_cast<Gtk::Window&>(*get_parent()));
  end_modal_handler_id = modal->destroy.connect
    (SigC::slot(this, &my_color_selection::end_search_handler));
}

void
my_color_selection::end_search_handler()
{
  search_dialog* p_search = static_cast<search_dialog*>(modal);
  search_dialog::buttons b = p_search->get_button();
  switch (b) {
  case search_dialog::cancel:
    break;
  case search_dialog::find_first:
    search_string = p_search->get_result();
    find_result = p_clist->find_first(search_string.c_str());
    break;
  case search_dialog::find_next:
    search_string = p_search->get_result();
    find_result = p_clist->find_next(search_string.c_str());
    break;
  } /*switch*/
  end_modal_handler();
  if (!find_result) {
    string not_found = search_string + " not found.";
    p_echo->set_text(not_found.c_str());
  } /*if*/
}

struct stop_child_expansion_fun {
  stop_child_expansion_fun(Gtk::Box* b);
  void apply(Gtk::Box_Helpers::Child* kid);
  void operator() (Gtk::Box_Helpers::Child* kid);
  Gtk::Box* b;
};

stop_child_expansion_fun::stop_child_expansion_fun(Gtk::Box* b)
  : b(b)
{}

// this method is declared in gtk--/box.h but nowhere implemented :(
void
Gtk::Box_Helpers::Child::set_options(bool expand,bool fill,guint padding)
{
  gboolean dummy_expand;
  gboolean dummy_fill;
  guint dummy_padding;
  GtkPackType pk;
  gtk_box_query_child_packing
    (parent(), gtkobj()->widget, &dummy_expand, &dummy_fill, &dummy_padding, &pk);
  gtk_box_set_child_packing(parent(), gtkobj()->widget, expand, fill, padding, pk);
}

void
stop_child_expansion_fun::apply(Gtk::Box_Helpers::Child* kid)
{
  guint16 padding = kid->get_padding();
  kid->set_options(0, 0, padding);
}

void
stop_child_expansion_fun::operator() (Gtk::Box_Helpers::Child* kid)
{
  apply(kid);
}

my_color_selection::my_color_selection()
  : Gtk::ColorSelection(), modal(0), search_string("")
{
  static const char* rgb_file_name = "/etc/X11/rgb.txt";

  // sorting the list would be too slow without this
  set_update_policy(GTK_UPDATE_DELAYED);

  // before adding new children, prevent expansion of existing ones
  BoxList& kids = children();
  for_each(kids.begin(), kids.end(), stop_child_expansion_fun(this));

  // lay out the children here
  Gtk::HBox* p_hbox1 = manage(new Gtk::HBox(0, 10));
  Gtk::Button* p_grab_button = manage(new Gtk::Button("Grab..."));
  Gtk::Button* p_select_button = manage(new Gtk::Button("Set to selected")); 
  Gtk::Button* p_show_button = manage(new Gtk::Button("Show..."));
  
  p_hbox1->pack_start(*p_grab_button);
  p_hbox1->pack_start(*p_select_button);
  p_hbox1->pack_start(*p_show_button);
  pack_start(*p_hbox1, 0, 0, 5);

  Gtk::ScrolledWindow* p_scrolled = manage(new Gtk::ScrolledWindow);
  p_clist = manage(new my_clist(rgb_file_name));
  gdouble colors[4];
  get_color(colors);
  p_clist->fill(colors[0], colors[1], colors[2]);
  p_scrolled->add(*p_clist);
  p_scrolled->set_usize(-1, 180);
  p_scrolled->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  pack_start(*p_scrolled, 1, 1, 5);

  Gtk::HBox* p_hbox2 = manage(new Gtk::HBox(0, 10));
  Gtk::Button* p_search_button = manage(new Gtk::Button("Search..."));
  Gtk::Button* p_quit_button = manage(new Gtk::Button("Quit"));
  p_hbox2->pack_start(*p_search_button);
  p_hbox2->pack_start(*p_quit_button);
  pack_start(*p_hbox2, 0, 0, 5);

  p_echo = manage(new Gtk::Label(about));
  pack_start(*p_echo, 0, 0, 2);

  // set up callbacks
  p_quit_button->clicked.connect(SigC::slot(this, &my_color_selection::quit_handler));
  p_grab_button->clicked.connect(SigC::slot(this, &my_color_selection::grab_handler));
  p_show_button->clicked.connect(SigC::slot(this, &my_color_selection::show_handler));
  p_search_button->clicked.connect(SigC::slot(this, &my_color_selection::search_handler));
  p_select_button->clicked.connect(SigC::slot(this, &my_color_selection::list_select_handler));
  p_clist->key_press_event.connect(SigC::slot(this, &my_color_selection::enter_key_handler));
  p_clist->select_row.connect(SigC::slot(this, &my_color_selection::select_row_handler));
}

void
my_color_selection::set_color_and_notify(const gdouble* color)
{
  set_color(color);
  color_changed_impl();
}

void
my_color_selection::select_row_handler(gint row, gint column, GdkEvent* p_ev)
{
  p_echo->set_text(about);
}

gint
my_color_selection::enter_key_handler(GdkEventKey* p_key)
{
  if (p_key->keyval == GDK_Return || p_key->keyval == GDK_KP_Enter) {
    const gdouble* p_selected = p_clist->get_selected_color();
    if (p_selected != 0) {
      set_color_and_notify(p_selected);
      p_echo->set_text(about);
    } else {
      p_echo->set_text("Please select a color from the list first.");
    } /*if*/
    return 1;
  } else {
    return 0;
  } /*if*/
}

void
my_color_selection::list_select_handler()
{
  const gdouble* p_selected = p_clist->get_selected_color();
  if (p_selected != 0) {
    set_color_and_notify(p_selected);
    p_echo->set_text(about);
  } else {
    p_echo->set_text("Please select a color from the list first.");
  } /*if*/
}

void
my_color_selection::color_changed_impl()
{
  static gdouble last_colors[4] = {1.0, 1.0, 1.0, 0.0};
  gdouble colors[4];
  get_color(colors);
  if (colors[0] != last_colors[0] || colors[1] != last_colors[1] || colors[2] != last_colors[2]) {
    last_colors[0] = colors[0];
    last_colors[1] = colors[1];
    last_colors[2] = colors[2];
    p_clist->color_change(colors[0], colors[1], colors[2]);
  } /*if*/
}
